@@ -1,6 +1,6 @@
-#  ATContentTypes http://sf.net/projects/collective/
+#  ATContentTypes http://plone.org/products/atcontenttypes/
 #  Archetypes reimplementation of the CMF core types
-#  Copyright (c) 2003-2005 AT Content Types development team
+#  Copyright (c) 2003-2006 AT Content Types development team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 
 """
-__author__  = 'Christian Heimes <ch@comlounge.net>'
+__author__  = 'Christian Heimes <tiran@cheimes.de>'
 __docformat__ = 'restructuredtext'
 __old_name__ = 'Products.ATContentTypes.types.ATEvent'
 
@@ -39,7 +39,7 @@ from Products.Archetypes.public import StringField
 from Products.Archetypes.public import TextField
 from Products.Archetypes.public import CalendarWidget
 from Products.Archetypes.public import LinesWidget
-from Products.Archetypes.public import MultiSelectionWidget
+from Products.Archetypes.public import KeywordWidget
 from Products.Archetypes.public import RichWidget
 from Products.Archetypes.public import StringWidget
 from Products.Archetypes.public import RFC822Marshaller
@@ -103,14 +103,13 @@ ATEventSchema = ATContentTypeSchema.copy() + Schema((
               primary=True,
               storage = AnnotationStorage(migrate=True),
               validators = ('isTidyHtmlWithCleanup',),
-              #validators = ('isTidyHtml',),
               default_content_type = zconf.ATEvent.default_content_type,
               default_output_type = 'text/x-html-safe',
               allowable_content_types = zconf.ATEvent.allowed_content_types,
               widget = RichWidget(
                         description = "",
                         description_msgid = "help_event_announcement",
-                        label = "Event Announcement",
+                        label = "Event body text",
                         label_msgid = "label_event_announcement",
                         rows = 25,
                         i18n_domain = "plone",
@@ -127,12 +126,11 @@ ATEventSchema = ATContentTypeSchema.copy() + Schema((
                                   i18n_domain="plone")),
 
     LinesField('eventType',
-               required=True,
+               required=False,
                searchable=True,
                write_permission = ChangeEvents,
-               vocabulary = 'getEventTypes',
                languageIndependent=True,
-               widget = MultiSelectionWidget(
+               widget = KeywordWidget(
                         size = 6,
                         description="",
                         description_msgid = "help_event_type",
@@ -145,7 +143,7 @@ ATEventSchema = ATContentTypeSchema.copy() + Schema((
                 searchable=True,
                 accessor='event_url',
                 write_permission = ChangeEvents,
-                validators = ('isURL',),
+                validators=('isURL',),
                 widget = StringWidget(
                         description = ("Web address with more info about the event. "
                                        "Add http:// for external links."),
@@ -184,9 +182,6 @@ ATEventSchema = ATContentTypeSchema.copy() + Schema((
                 searchable=True,
                 accessor='contact_phone',
                 write_permission = ChangeEvents,
-                # XXX disabled for now, see
-                # https://sourceforge.net/tracker/index.php?func=detail&aid=974102&group_id=55262&atid=645337
-                #validators = ('isInternationalPhoneNumber',),
                 validators= (),
                 widget = StringWidget(
                         description = "",
@@ -259,7 +254,7 @@ class ATEvent(ATCTContent, CalendarSupportMixin, HistoryAwareMixin):
         if type(value) is StringType:
             v = (value, )
         elif value:
-            v = value[0]
+            v = value
         else:
             v = ()
 
@@ -270,9 +265,9 @@ class ATEvent(ATCTContent, CalendarSupportMixin, HistoryAwareMixin):
     def getEventTypes(self):
         """fetch a list of the available event types from the vocabulary
         """
-        metatool = getToolByName(self, "portal_metadata")
-        events = metatool.listAllowedSubjects(content_type = "Event")
-        return events
+        f = self.getField('eventType')
+        result = self.collectKeywords('eventType', f.accessor)
+        return tuple(result)
 
     security.declarePrivate('cmf_edit')
     def cmf_edit(self, title=None, description=None, eventType=None,
@@ -321,6 +316,10 @@ class ATEvent(ATCTContent, CalendarSupportMixin, HistoryAwareMixin):
 
         End date must be after start date
         """
+        if 'startDate' in errors or 'endDate' in errors:
+            # No point in validating bad input
+            return
+        
         rstartDate = REQUEST.get('startDate', None)
         rendDate = REQUEST.get('endDate', None)
 
@@ -376,7 +375,7 @@ class ATEvent(ATCTContent, CalendarSupportMixin, HistoryAwareMixin):
         elif isinstance(other, DateTime):
             return cmp(self.start(), other)
         else:
-            # XXX come up with a nice cmp for types
+            # TODO come up with a nice cmp for types
             return cmp(self.Title(), other)
 
     def __hash__(self):
