@@ -1,6 +1,6 @@
-#  ATContentTypes http://sf.net/projects/collective/
+#  ATContentTypes http://plone.org/products/atcontenttypes/
 #  Archetypes reimplementation of the CMF core types
-#  Copyright (c) 2003-2005 AT Content Types development team
+#  Copyright (c) 2003-2006 AT Content Types development team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 """Test module for bug reports
 """
 
-__author__ = 'Christian Heimes <ch@comlounge.net>'
+__author__ = 'Christian Heimes <tiran@cheimes.de>'
 __docformat__ = 'restructuredtext'
 
 import os, sys
@@ -28,12 +28,10 @@ if __name__ == '__main__':
 
 from Testing import ZopeTestCase # side effect import. leave it here.
 from Products.ATContentTypes.tests import atcttestcase
+from Products.validation.interfaces.IValidator import IValidationChain
+from Products.ATContentTypes.content.schemata import ATContentTypeSchema
 
-from Products.Archetypes.public import *
-import time
-
-from Interface.Verify import verifyObject
-
+from Products.Archetypes.atapi import *
 
 tests = []
 
@@ -48,32 +46,32 @@ class TestBugs(atcttestcase.ATCTSiteTestCase):
 
     def test_reinstall_keeps_obj_wf_state(self):
         wf = self.wf
-        self.folder.invokeFactory('Document', 'testdoc', title="test doc", 
+        self.folder.invokeFactory('Document', 'testdoc', title="test doc",
                                   text="test body")
         obj = self.folder.testdoc
-        
+
         state = wf.getInfoFor(obj, 'review_state', default=None)
         self.failUnlessEqual(state, 'visible')
-        
+
         self.qi.reinstallProducts(('ATContentTypes',))
-        
+
         state = wf.getInfoFor(obj, 'review_state', default=None)
         self.failUnlessEqual(state, 'visible')
-        
+
         wf.doActionFor(obj, 'submit')
-        
+
         state = wf.getInfoFor(obj, 'review_state', default=None)
         self.failUnlessEqual(state, 'pending')
 
         self.qi.reinstallProducts(('ATContentTypes',))
-        
+
         state = wf.getInfoFor(obj, 'review_state', default=None)
         self.failUnlessEqual(state, 'pending')
 
     def test_wfmapping(self):
         default = ('plone_workflow',)
         folder = ('folder_workflow',)
-        
+
         mapping = {
             'Document' : default,
             'Event' : default,
@@ -86,24 +84,36 @@ class TestBugs(atcttestcase.ATCTSiteTestCase):
             'News Item' : default,
             'Topic' : folder,
             }
-        
+
         for pt, wf in mapping.items():
             pwf = self.wf.getChainFor(pt)
             self.failUnlessEqual(pwf, wf, (pt, pwf, wf))
-    
+
     def test_reinstall_keeps_wfmapping(self):
         pt = 'Document'
         wf = ('folder_workflow',)
-        
-        self.wf.setChainForPortalTypes(pt, wf)
+
+        self.wf.setChainForPortalTypes((pt,), wf)
         pwf = self.wf.getChainFor(pt)
         self.failUnlessEqual(pwf, wf, (pt, pwf, wf))
-        
+
         self.qi.reinstallProducts(('ATContentTypes',))
-        
+
         pwf = self.wf.getChainFor(pt)
         self.failUnlessEqual(pwf, wf, (pt, pwf, wf))
 
+    def test_striphtmlbug(self):
+        # Test for Plone tracker #4944
+        self.folder.invokeFactory('Document', 'document')
+        d = getattr(self.folder, 'document')
+        d.setTitle("HTML end tags start with </ and end with >")
+        self.assertEqual(d.Title(), "HTML end tags start with </ and end with >")
+
+    def test_validation_layer_from_id_field_from_base_schema_was_initialized(self):
+        field = ATContentTypeSchema['id']
+        self.failUnless(IValidationChain.isImplementedBy(field.validators))
+
+
 tests.append(TestBugs)
 
 
