@@ -20,28 +20,32 @@ from Products.Archetypes.Storage import AttributeStorage
 from Products.Archetypes.Widget import IdWidget
 from Products.Archetypes.Widget import StringWidget
 from Products.Archetypes.Marshall import RFC822Marshaller
-from Products.Archetypes.interfaces.base import IBaseObject
-from Products.Archetypes.interfaces.base import IBaseUnit
+from Products.Archetypes.interfaces.base import IBaseObject as z2IBaseObject
+from Products.Archetypes.interfaces.base import IBaseUnit as z2IBaseUnit
 from Products.Archetypes.interfaces.field import IFileField
 from Products.Archetypes.config import ATTRIBUTE_SECURITY
+from Products.Archetypes.config import RENAME_AFTER_CREATION_ATTEMPTS
 from Products.Archetypes.ArchetypeTool import getType
 from Products.Archetypes.ArchetypeTool import _guessPackage
 
 from AccessControl import ClassSecurityInfo
 from AccessControl import Unauthorized
+from AccessControl.Permissions import copy_or_move as permission_copy_or_move
 from Acquisition import Implicit
 from Acquisition import aq_base
 from Acquisition import aq_acquire
 from Acquisition import aq_inner
 from Acquisition import aq_parent
+from Acquisition import ImplicitAcquisitionWrapper
 from Acquisition import ExplicitAcquisitionWrapper
 from Acquisition import Explicit
 
 from ComputedAttribute import ComputedAttribute
 from OFS.ObjectManager import ObjectManager
 from ZODB.POSException import ConflictError
+import transaction
 
-from Products.CMFCore  import CMFCorePermissions
+from Products.CMFCore import permissions
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.utils import _checkPermission as checkPerm
 
@@ -51,6 +55,12 @@ from types import TupleType, ListType, UnicodeType
 from ZPublisher import xmlrpc
 from webdav.NullResource import NullResource
 
+from Products.Archetypes.interfaces import IBaseObject
+from zope.interface import implements, Interface
+from zope import event
+from zope.app.event import objectevent
+from zope.component import queryMultiAdapter
+
 _marker = []
 
 class AttributeValidator(Implicit):
@@ -95,6 +105,7 @@ content_type = Schema((
         required=0, # Still actually required, but the widget will
                     # supply the missing value on non-submits
         mode='rw',
+        permission=permission_copy_or_move,
         accessor='getId',
         mutator='setId',
         default=None,
@@ -146,12 +157,14 @@ class BaseObject(Referenceable):
     typeDescription = ''
     _at_rename_after_creation = False # rename object according to title?
 
-    __implements__ = (IBaseObject, ) + Referenceable.__implements__
+    __implements__ = (z2IBaseObject, ) + Referenceable.__implements__
+
+    implements(IBaseObject)
 
     def __init__(self, oid, **kwargs):
         self.id = oid
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'initializeArchetype')
     def initializeArchetype(self, **kwargs):
         """Called by the generated addXXX factory in types tool.
@@ -161,14 +174,13 @@ class BaseObject(Referenceable):
             self.markCreationFlag()
             self.setDefaults()
             if kwargs:
+                kwargs['_initializing_'] = True
                 self.edit(**kwargs)
             self._signature = self.Schema().signature()
-        except ConflictError:
+        except (ConflictError, KeyboardInterrupt):
             raise
         except:
             log_exc()
-            #_default_logger.log_exc()
-            #raise
 
     security.declarePrivate('manage_afterAdd')
     def manage_afterAdd(self, item, container):
@@ -195,7 +207,7 @@ class BaseObject(Referenceable):
     def cleanupLayers(self, item=None, container=None):
         self.Schema().cleanupLayers(self, item, container)
 
-    security.declareProtected(CMFCorePermissions.View, 'title_or_id')
+    security.declareProtected(permissions.View, 'title_or_id')
     def title_or_id(self):
         """Returns the title if it is not blank and the id otherwise.
         """
@@ -205,13 +217,13 @@ class BaseObject(Referenceable):
 
         return self.getId()
 
-    security.declareProtected(CMFCorePermissions.View, 'getId')
+    security.declareProtected(permissions.View, 'getId')
     def getId(self):
         """Gets the object id.
         """
         return self.id
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setId')
+    security.declareProtected(permissions.ModifyPortalContent, 'setId')
     def setId(self, value):
         """Sets the object id.
         """
@@ -223,7 +235,7 @@ class BaseObject(Referenceable):
                 parent.manage_renameObject(self.id, value)
             self._setId(value)
 
-    security.declareProtected(CMFCorePermissions.View, 'Type')
+    security.declareProtected(permissions.View, 'Type')
     def Type(self):
         """Dublin Core element - Object type.
 
@@ -237,13 +249,13 @@ class BaseObject(Referenceable):
                 return ti.Title()
         return self.meta_type
 
-    security.declareProtected(CMFCorePermissions.View, 'getField')
+    security.declareProtected(permissions.View, 'getField')
     def getField(self, key, wrapped=False):
         """Returns a field object.
         """
         return self.Schema().get(key)
 
-    security.declareProtected(CMFCorePermissions.View, 'getWrappedField')
+    security.declareProtected(permissions.View, 'getWrappedField')
     def getWrappedField(self, key):
         """Gets a field by id which is explicitly wrapped.
 
@@ -251,20 +263,21 @@ class BaseObject(Referenceable):
         """
         return ExplicitAcquisitionWrapper(self.getField(key), self)
 
-    security.declareProtected(CMFCorePermissions.View, 'getDefault')
+    security.declareProtected(permissions.View, 'getDefault')
     def getDefault(self, field):
         """Return the default value of a field.
         """
         field = self.getField(field)
         return field.getDefault(self)
 
-    security.declareProtected(CMFCorePermissions.View, 'isBinary')
+    security.declareProtected(permissions.View, 'isBinary')
     def isBinary(self, key):
         """Return wether a field contains binary data.
         """
-        element = getattr(self, key, None)
-        if element and shasattr(element, 'isBinary'):
-            return element.isBinary()
+        field = self.getField(key)
+        if IFileField.isImplementedBy(field):
+            value = field.getBaseUnit(self)
+            return value.isBinary()
         mimetype = self.getContentType(key)
         if mimetype and shasattr(mimetype, 'binary'):
             return mimetype.binary
@@ -272,14 +285,14 @@ class BaseObject(Referenceable):
             return 0
         return 1
 
-    security.declareProtected(CMFCorePermissions.View, 'isTransformable')
+    security.declareProtected(permissions.View, 'isTransformable')
     def isTransformable(self, name):
         """Returns wether a field is transformable.
         """
         field = self.getField(name)
         return isinstance(field, TextField) or not self.isBinary(name)
 
-    security.declareProtected(CMFCorePermissions.View, 'widget')
+    security.declareProtected(permissions.View, 'widget')
     def widget(self, field_name, mode="view", field=None, **kwargs):
         """Returns the rendered widget.
         """
@@ -289,7 +302,7 @@ class BaseObject(Referenceable):
         return renderer.render(field_name, mode, widget, self, field=field,
                                **kwargs)
 
-    security.declareProtected(CMFCorePermissions.View, 'getFilename')
+    security.declareProtected(permissions.View, 'getFilename')
     def getFilename(self, key=None):
         """Returns the filename from a field.
         """
@@ -305,7 +318,7 @@ class BaseObject(Referenceable):
 
         return value
 
-    security.declareProtected(CMFCorePermissions.View, 'getContentType')
+    security.declareProtected(permissions.View, 'getContentType')
     def getContentType(self, key=None):
         """Returns the content type from a field.
         """
@@ -322,15 +335,16 @@ class BaseObject(Referenceable):
         return value
 
     # Backward compatibility
-    security.declareProtected(CMFCorePermissions.View, 'content_type')
+    # Note: ComputedAttribute should never be protected by a security
+    # declaration! See http://dev.plone.org/archetypes/ticket/712
     content_type = ComputedAttribute(getContentType, 1)
 
     # XXX Where's get_content_type comes from??? There's no trace at both
     # Zope and CMF. It should be removed ASAP!
-    security.declareProtected(CMFCorePermissions.View, 'get_content_type')
+    security.declareProtected(permissions.View, 'get_content_type')
     get_content_type = getContentType
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'setContentType')
     def setContentType(self, value, key=None):
         """Sets the content type of a field.
@@ -343,8 +357,7 @@ class BaseObject(Referenceable):
         if field and IFileField.isImplementedBy(field):
             field.setContentType(self, value)
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
-                              'setFilename')
+    security.declareProtected(permissions.ModifyPortalContent, 'setFilename')
     def setFilename(self, value, key=None):
         """Sets the filename of a field.
         """
@@ -356,7 +369,7 @@ class BaseObject(Referenceable):
         if field and IFileField.isImplementedBy(field):
             field.setFilename(self, value)
 
-    security.declareProtected(CMFCorePermissions.View, 'getPrimaryField')
+    security.declareProtected(permissions.View, 'getPrimaryField')
     def getPrimaryField(self):
         """The primary field is some object that responds to
         PUT/manage_FTPget events.
@@ -366,7 +379,7 @@ class BaseObject(Referenceable):
             return fields[0]
         return None
 
-    security.declareProtected(CMFCorePermissions.View, 'get_portal_metadata')
+    security.declareProtected(permissions.View, 'get_portal_metadata')
     def get_portal_metadata(self, field):
         """Returns the portal_metadata for a field.
         """
@@ -375,7 +388,8 @@ class BaseObject(Referenceable):
         try:
             spec = pmt.getElementSpec(field.accessor)
             policy = spec.getPolicy(self.portal_type)
-        except ConflictError:
+
+        except (ConflictError, KeyboardInterrupt):
             raise
         except:
             log_exc()
@@ -387,7 +401,7 @@ class BaseObject(Referenceable):
         return DisplayList(map(lambda x: (x,x), policy.allowedVocabulary())), \
                policy.enforceVocabulary()
 
-    security.declareProtected(CMFCorePermissions.View, 'Vocabulary')
+    security.declareProtected(permissions.View, 'Vocabulary')
     def Vocabulary(self, key):
         """Returns the vocabulary for a specified field.
         """
@@ -440,19 +454,23 @@ class BaseObject(Referenceable):
         """
         self.Schema().setDefaults(self)
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'update')
+    security.declareProtected(permissions.ModifyPortalContent, 'update')
     def update(self, **kwargs):
         """Changes the values of the field and reindex the object.
         """
+        initializing = kwargs.get('_initializing_', False)
+        if initializing:
+            del kwargs['_initializing_']
         self.Schema().updateAll(self, **kwargs)
         self._p_changed = 1
-        self.reindexObject()
+        if not initializing:
+            # Avoid double indexing during initialization.
+            self.reindexObject()
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
+    security.declareProtected(permissions.ModifyPortalContent, 'edit')
     edit = update
 
-    security.declareProtected(CMFCorePermissions.View,
-                              'validate_field')
+    security.declareProtected(permissions.View, 'validate_field')
     def validate_field(self, name, value, errors):
         """Field's validate hook.
 
@@ -474,15 +492,15 @@ class BaseObject(Referenceable):
 
     ## Pre/post validate hooks that will need to write errors
     ## into the errors dict directly using errors[fieldname] = ""
-    security.declareProtected(CMFCorePermissions.View, 'pre_validate')
+    security.declareProtected(permissions.View, 'pre_validate')
     def pre_validate(self, REQUEST=None, errors=None):
         pass
 
-    security.declareProtected(CMFCorePermissions.View, 'post_validate')
+    security.declareProtected(permissions.View, 'post_validate')
     def post_validate(self, REQUEST=None, errors=None):
         pass
 
-    security.declareProtected(CMFCorePermissions.View, 'validate')
+    security.declareProtected(permissions.View, 'validate')
     def validate(self, REQUEST=None, errors=None, data=None, metadata=None):
         """Validates the form data from the request.
         """
@@ -496,7 +514,7 @@ class BaseObject(Referenceable):
         self.post_validate(REQUEST, errors)
         return errors
 
-    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
+    security.declareProtected(permissions.View, 'SearchableText')
     def SearchableText(self):
         """All fields marked as 'searchable' are concatenated together
         here for indexing purpose.
@@ -506,7 +524,7 @@ class BaseObject(Referenceable):
         for field in self.Schema().fields():
             if not field.searchable:
                 continue
-            method = field.getAccessor(self)
+            method = field.getIndexAccessor(self)
             try:
                 datum =  method(mimetype="text/plain")
             except TypeError:
@@ -514,7 +532,7 @@ class BaseObject(Referenceable):
                 # handle the mimetype argument
                 try:
                     datum =  method()
-                except ConflictError:
+                except (ConflictError, KeyboardInterrupt):
                     raise
                 except:
                     continue
@@ -530,9 +548,11 @@ class BaseObject(Referenceable):
                 elif type_datum in STRING_TYPES:
                     if type_datum is UnicodeType:
                         datum = datum.encode(charset)
-                    datum = "%s %s" % (datum, vocab.getValue(datum, ''), )
+                    value = vocab.getValue(datum, '')
+                    if type(value) is UnicodeType:
+                        value = value.encode(charset)
+                    datum = "%s %s" % (datum, value, )
 
-                # XXX: We really need an unicode policy !
                 type_datum = type(datum)
                 if type_datum is UnicodeType:
                     datum = datum.encode(charset)
@@ -541,7 +561,7 @@ class BaseObject(Referenceable):
         data = ' '.join(data)
         return data
 
-    security.declareProtected(CMFCorePermissions.View, 'getCharset')
+    security.declareProtected(permissions.View, 'getCharset')
     def getCharset(self):
         """Returns the site default charset, or utf-8.
         """
@@ -552,7 +572,7 @@ class BaseObject(Referenceable):
                 return site_properties.getProperty('default_charset')
         return 'utf-8'
 
-    security.declareProtected(CMFCorePermissions.View, 'get_size')
+    security.declareProtected(permissions.View, 'get_size')
     def get_size(self):
         """Used for FTP and apparently the ZMI now too.
         """
@@ -600,10 +620,22 @@ class BaseObject(Referenceable):
                 # exist just bail out.
                 continue
 
-            widget = field.widget
-            result = widget.process_form(self, field, form,
-                                         empty_marker=_marker)
-            if result is _marker or result is None: continue
+            result = field.widget.process_form(self, field, form,
+                                               empty_marker=_marker)
+            try:
+                # Pass validating=False to inform the widget that we
+                # aren't in the validation phase, IOW, the returned
+                # data will be forwarded to the storage
+                result = field.widget.process_form(self, field, form,
+                                                   empty_marker=_marker,
+                                                   validating=False)
+            except TypeError:
+                # Support for old-style process_form methods
+                result = field.widget.process_form(self, field, form,
+                                                   empty_marker=_marker)
+
+            if result is _marker or result is None:
+                continue
 
             # Set things by calling the mutator
             mutator = field.getMutator(self)
@@ -613,8 +645,7 @@ class BaseObject(Referenceable):
 
         self.reindexObject()
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
-                              'processForm')
+    security.declareProtected(permissions.ModifyPortalContent, 'processForm')
     def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
         """Processes the schema looking for data in the form.
         """
@@ -624,24 +655,26 @@ class BaseObject(Referenceable):
         self.unmarkCreationFlag()
         if self._at_rename_after_creation and is_new_object:
             self._renameAfterCreation(check_auto_id=True)
-        
+
         # Post create/edit hooks
         if is_new_object:
             self.at_post_create_script()
         else:
             self.at_post_edit_script()
 
+        event.notify(objectevent.ObjectModifiedEvent(self))
+
     # This method is only called once after object creation.
-    security.declareProtected(CMFCorePermissions.View, 'at_post_create_script')
+    security.declarePrivate('at_post_create_script')
     def at_post_create_script(self):
         pass
 
     # This method is called after every subsequent edit
-    security.declareProtected(CMFCorePermissions.View, 'at_post_edit_script')
+    security.declarePrivate('at_post_edit_script')
     def at_post_edit_script(self):
         pass
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'markCreationFlag')
     def markCreationFlag(self):
         """Sets flag on the instance to indicate that the object hasn't been
@@ -661,7 +694,7 @@ class BaseObject(Referenceable):
             if meth in ('GET', 'POST'):
                 self._at_creation_flag = True
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'unmarkCreationFlag')
     def unmarkCreationFlag(self):
         """Removes the creation flag.
@@ -669,16 +702,16 @@ class BaseObject(Referenceable):
         if shasattr(aq_inner(self), '_at_creation_flag'):
             self._at_creation_flag = False
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'checkCreationFlag')
     def checkCreationFlag(self):
         """Returns True if the object has been fully saved, False otherwise.
         """
         return getattr(aq_base(self), '_at_creation_flag', False)
 
-    security.declarePrivate('_renameAfterCreation')
-    def _renameAfterCreation(self, check_auto_id=False):
-        """Renames an object like its normalized title.
+    def generateNewId(self):
+        """Suggest an id for this object.
+        This id is used when automatically renaming an object after creation.
         """
         plone_tool = getToolByName(self, 'plone_utils', None)
         if plone_tool is None or not shasattr(plone_tool, 'normalizeString'):
@@ -689,32 +722,72 @@ class BaseObject(Referenceable):
         title = self.Title()
         if not title:
             # Can't work w/o a title
-            return False
+            return None
 
+        return plone_tool.normalizeString(title)
+
+    security.declarePrivate('_renameAfterCreation')
+    def _renameAfterCreation(self, check_auto_id=False):
+        """Renames an object like its normalized title.
+        """
         old_id = self.getId()
         if check_auto_id and not self._isIDAutoGenerated(old_id):
             # No auto generated id
             return False
 
-        new_id = plone_tool.normalizeString(title)
-        invalid_id = False
+        new_id = self.generateNewId()
+        if new_id is None:
+            return False
+
+        invalid_id = True
         check_id = getattr(self, 'check_id', None)
         if check_id is not None:
             invalid_id = check_id(new_id, required=1)
-        else:
-            # If check_id is not available just look for conflicting ids
-            parent = aq_parent(aq_inner(self))
-            invalid_id = new_id in parent.objectIds()
+
+        # If check_id told us no, or if it was not found, make sure we have an
+        # id unique in the parent folder.
+        if invalid_id:
+            unique_id = self._findUniqueId(new_id)
+            if unique_id is not None:
+                if check_id is None or check_id(new_id, required=1):
+                    new_id = unique_id
+                    invalid_id = False
 
         if not invalid_id:
             # Can't rename without a subtransaction commit when using
             # portal_factory!
-            get_transaction().commit(1)
+            transaction.savepoint(optimistic=True)
             self.setId(new_id)
             return new_id
 
         return False
 
+    security.declarePrivate('_findUniqueId')
+    def _findUniqueId(self, id):
+        """Find a unique id in the parent folder, based on the given id, by
+        appending -n, where n is a number between 1 and the constant
+        RENAME_AFTER_CREATION_ATTEMPTS, set in config.py. If no id can be
+        found, return None.
+        """
+        check_id = getattr(self, 'check_id', None)
+        if check_id is None:
+            parent = aq_parent(aq_inner(self))
+            parent_ids = parent.objectIds()
+            check_id = lambda id, required: id in parent_ids
+
+        invalid_id = check_id(id, required=1)
+        if not invalid_id:
+            return id
+
+        idx = 1
+        while idx <= RENAME_AFTER_CREATION_ATTEMPTS:
+            new_id = "%s-%d" % (id, idx)
+            if not check_id(new_id, required=1):
+                return new_id
+            idx += 1
+
+        return None
+
     security.declarePrivate('_isIDAutoGenerated')
     def _isIDAutoGenerated(self, id):
         """Avoid busting setDefaults if we don't have a proper acquisition
@@ -730,12 +803,18 @@ class BaseObject(Referenceable):
             return script(id)
         return False
 
-    security.declareProtected(CMFCorePermissions.View, 'Schemata')
+    security.declareProtected(permissions.View, 'Schemata')
     def Schemata(self):
         """Returns the Schemata for the Object.
         """
         return getSchemata(self)
 
+    def Schema(self):
+        """Return a (wrapped) schema instance for this object instance.
+        """
+        schema = self.schema
+        return ImplicitAcquisitionWrapper(schema, self)
+
     security.declarePrivate('_isSchemaCurrent')
     def _isSchemaCurrent(self):
         """Determines whether the current object's schema is up to date."""
@@ -764,6 +843,7 @@ class BaseObject(Referenceable):
 
         # Read all the old values into a dict
         values = {}
+        mimes = {}
         for f in new_schema.fields():
             name = f.getName()
             if name in excluded_fields: continue
@@ -774,6 +854,9 @@ class BaseObject(Referenceable):
                 if out != None:
                     print >> out, ('Unable to get %s.%s'
                                    % (str(self.getId()), name))
+            else:
+                if shasattr(f, 'getContentType'):
+                    mimes[name] = f.getContentType(self)
 
         obj_class = self.__class__
         current_class = getattr(sys.modules[self.__module__],
@@ -799,9 +882,8 @@ class BaseObject(Referenceable):
             name = f.getName()
             kw = {}
             if name not in excluded_fields and values.has_key(name):
-                # XXX Commented out because we are trying to directly use
-                # the new base unit
-                # kw['mimetype'] = f.getContentType(self)
+                if mimes.has_key(name):
+                    kw['mimetype'] = mimes[name]
                 try:
                     self._migrateSetValue(name, values[name], **kw)
                 except ValueError:
@@ -827,7 +909,8 @@ class BaseObject(Referenceable):
             try:
                 if IFileField.isImplementedBy(field):
                     return field.getBaseUnit(self)
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -837,7 +920,8 @@ class BaseObject(Referenceable):
                 editAccessor = field.getEditAccessor(self)
                 if editAccessor:
                     return editAccessor()
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -847,7 +931,8 @@ class BaseObject(Referenceable):
                 accessor = field.getAccessor(self)
                 if accessor:
                     return accessor()
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -862,7 +947,8 @@ class BaseObject(Referenceable):
             # Because of this line value = value.replace(',','.')
             try:
                 return self[field.getName()]
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -876,7 +962,8 @@ class BaseObject(Referenceable):
                 editAccessor = new_field.getEditAccessor(self)
                 if editAccessor:
                     return editAccessor()
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -886,7 +973,8 @@ class BaseObject(Referenceable):
                 accessor = new_field.getAccessor(self)
                 if accessor:
                     return accessor()
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -894,7 +982,8 @@ class BaseObject(Referenceable):
             # Still no luck -- try to get the value directly using the new name
             try:
                 return self[new_field.getName()]
-            except ConflictError:
+
+            except (ConflictError, KeyboardInterrupt):
                 raise
             except:
                 pass
@@ -909,7 +998,8 @@ class BaseObject(Referenceable):
 
     security.declarePrivate('_migrateSetValue')
     def _migrateSetValue(self, name, value, old_schema=None, **kw):
-        """Try to set an object value using a variety of methods."""
+        """Try to set an object value using a variety of methods.
+        """
         schema = self.Schema()
         # Migrate pre-AT 1.3 schemas.
         schema = fixSchema(schema)
@@ -922,7 +1012,8 @@ class BaseObject(Referenceable):
                     args = [value,]
                     mapply(mutator, *args, **kw)
                     return
-                except ConflictError:
+
+                except (ConflictError, KeyboardInterrupt):
                     raise
                 except:
                     log_exc()
@@ -933,7 +1024,7 @@ class BaseObject(Referenceable):
                 return
         raise ValueError, 'name = %s, value = %s' % (name, value)
 
-    security.declareProtected(CMFCorePermissions.View, 'isTemporary')
+    security.declareProtected(permissions.View, 'isTemporary')
     def isTemporary(self):
         """Checks to see if we are created as temporary object by
         portal factory.
@@ -942,7 +1033,7 @@ class BaseObject(Referenceable):
         return shasattr(parent, 'meta_type') and \
                parent.meta_type == 'TempFolder'
 
-    security.declareProtected(CMFCorePermissions.View,
+    security.declareProtected(permissions.View,
                               'getFolderWhenPortalFactory')
     def getFolderWhenPortalFactory(self):
         """Returns the folder where this object was created temporarily.
@@ -973,7 +1064,7 @@ class BaseObject(Referenceable):
     # Those objects are specific to a session.
     #
 
-    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
+    security.declareProtected(permissions.ModifyPortalContent,
                               'addSubObjects')
     def addSubObjects(self, objects, REQUEST=None):
         """Adds a dictionary of objects to a volatile attribute.
@@ -986,7 +1077,7 @@ class BaseObject(Referenceable):
             for name, obj in objects.items():
                 storage[name] = aq_base(obj)
 
-    security.declareProtected(CMFCorePermissions.View, 'getSubObject')
+    security.declareProtected(permissions.View, 'getSubObject')
     def getSubObject(self, name, REQUEST, RESPONSE=None):
         """Gets a dictionary of objects from a volatile attribute.
         """
@@ -1020,24 +1111,35 @@ class BaseObject(Referenceable):
         # Logic from "ZPublisher.BaseRequest.BaseRequest.traverse"
         # to check whether this is a browser request
         if (len(REQUEST.get('TraversalRequestNameStack', ())) == 0 and
-            not (method in ('GET', 'POST') and not
+            not (method in ('GET', 'HEAD', 'POST') and not
                  isinstance(RESPONSE, xmlrpc.Response))):
             if shasattr(self, name):
                 target = getattr(self, name)
         else:
-            # We are allowed to acquire
-            target = getattr(self, name, None)
+            if shasattr(self, name): # attributes of self come first
+                target = getattr(self, name)
+            else: # then views
+                target = queryMultiAdapter((self, REQUEST), Interface, name)
+                if target is not None:
+                    # We don't return the view, we raise an
+                    # AttributeError instead (below)
+                    target = None
+                else: # then acquired attributes
+                    target = getattr(self, name, None)
+
         if target is not None:
             return target
-        if (method in ('PUT', 'MKCOL') and not
-            isinstance(RESPONSE, xmlrpc.Response)):
+        elif (method not in ('GET', 'POST') and not
+              isinstance(RESPONSE, xmlrpc.Response) and
+              REQUEST.maybe_webdav_client):
             return NullResource(self, name, REQUEST).__of__(self)
-
-        # Nothing has been found. Raise an AttributeError and be done with it.
-        raise AttributeError(name)
+        else:
+            # Raising AttributeError will look up views for us
+            raise AttributeError(name)
 
 InitializeClass(BaseObject)
 
+
 class Wrapper(Explicit):
     """Wrapper object for access to sub objects."""
     __allow_access_to_unprotected_subobjects__ = 1
