@@ -73,7 +73,7 @@ class TypesWidget(macrowidget, Base):
             True/1:  'visible'
             False/0: 'invisible'
             -1:      'hidden'
-            
+
         visible: The field is shown in the view/edit screen
         invisible: The field is skipped when rendering the view/edit screen
         hidden: The field is added as <input type="hidden" />
@@ -122,7 +122,7 @@ class TypesWidget(macrowidget, Base):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """Basic impl for form processing in a widget"""
         value = form.get(field.getName(), empty_marker)
         if value is empty_marker:
@@ -327,9 +327,9 @@ class TextAreaWidget(TypesWidget):
         'format': 0,
         'append_only': False,
         'divider':"\n\n========================\n\n",
-        'timestamp': True,
+        'timestamp': False,
         'maxlength' : False,
-        'helper_js': ('widgets/js/textcount.js',),        
+        'helper_js': ('widgets/js/textcount.js',),
         })
 
     security = ClassSecurityInfo()
@@ -337,7 +337,7 @@ class TextAreaWidget(TypesWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """handle text formatting"""
         text_format = None
         value = None
@@ -364,14 +364,14 @@ class TextAreaWidget(TypesWidget):
         if getattr(field.widget, 'append_only', None):
             if field.getEditAccessor(instance)():
                 if (value and not value.isspace()):
-                    
+
                     divider = field.widget.divider
-                    
+
                     # Add a datestamp along with divider if desired.
                     if getattr(field.widget, 'timestamp', None):
 
                         divider = "\n\n" + str(DateTime()) + divider
-                        
+
                     # using default_output_type caused a recursive transformation
                     # that sucked, thus mimetype= here to keep it in line
                     value = value + divider + \
@@ -404,8 +404,8 @@ class CalendarWidget(TypesWidget):
     _properties.update({
         'macro' : "widgets/calendar",
         'format' : '', # time.strftime string
-        'show_hm' : True, 
-        'show_ymd' : True, # False not supported by the plone templates yet
+        'show_hm' : True,
+        'show_ymd' : True, # Supported since Plone 2.5.5
         'starting_year' : 1999, # not supported by the plone templates yet
         'ending_year' : None, # not supported by the plone templates yet
         'future_years' : 5, # not supported by the plone templates yet
@@ -437,7 +437,7 @@ class MultiSelectionWidget(TypesWidget):
 
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """Basic impl for form processing in a widget"""
         value = form.get(field.getName(), empty_marker)
         if value is empty_marker:
@@ -466,7 +466,7 @@ class KeywordWidget(TypesWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """process keywords from form where this widget has a list of
         available keywords and any new ones"""
         name = field.getName()
@@ -497,7 +497,7 @@ class FileWidget(TypesWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """form processing that deals with binary data"""
 
         delete = form.get('%s_delete' % field.getName(), empty_marker)
@@ -537,7 +537,7 @@ class RichWidget(TypesWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """complex form processing, includes handling for text
         formatting and file objects"""
         # This is basically the old processing chain from base object
@@ -600,7 +600,7 @@ class IdWidget(TypesWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """the id might be hidden by the widget and not submitted"""
         value = form.get('id', empty_marker)
         if not value or value is empty_marker or not value.strip():
@@ -616,7 +616,8 @@ class RequiredIdWidget(IdWidget):
 
     # XXX
     security.declarePublic('process_form')
-    def process_form(self, instance, field, form, empty_marker=None):
+    def process_form(self, instance, field, form, empty_marker=None,
+                     emptyReturnsMarker=False, validating=True):
         """Override IdWidget.process_form to require id."""
         return TypesWidget.process_form(self, instance, field, form, empty_marker)
 
@@ -634,14 +635,13 @@ class ImageWidget(FileWidget):
     # XXX
     security.declarePublic('process_form')
     def process_form(self, instance, field, form, empty_marker=None,
-                     emptyReturnsMarker=False):
+                     emptyReturnsMarker=False, validating=True):
         """form processing that deals with image data (and its delete case)"""
         value = None
         ## check to see if the delete hidden was selected
         delete = form.get('%s_delete' % field.getName(), empty_marker)
         if delete=='delete': return "DELETE_IMAGE", {}
         if delete=='nochange' : return empty_marker
-        
 
         fileobj = form.get('%s_file' % field.getName(), empty_marker)
 
