@@ -27,6 +27,7 @@ from Products.CMFPlone import ToolNames
 from Products.CMFPlone.utils import transaction_note
 from Products.CMFPlone.utils import base_hasattr
 from Products.CMFPlone.utils import safe_hasattr
+from Products.CMFPlone.utils import postonly
 from Products.CMFPlone.interfaces import IBrowserDefault
 
 from OFS.SimpleItem import SimpleItem
@@ -360,7 +361,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
             return _icons[(category, id)]
         try:
             actionicons = getToolByName(self, 'portal_actionicons')
-            iconinfo = actionicons.getActionIcon(category, id)
+            iconinfo = actionicons.getActionIcon(category, id, context=self)
             icon = _icons.setdefault((category, id), iconinfo)
         except KeyError:
             if default is not _marker:
@@ -434,7 +435,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
 
     security.declareProtected(ManagePortal,
                               'changeOwnershipOf')
-    def changeOwnershipOf(self, object, userid, recursive=0):
+    def changeOwnershipOf(self, object, userid, recursive=0, REQUEST=None):
         """Changes the ownership of an object."""
         membership = getToolByName(self, 'portal_membership')
         acl_users = getattr(self, 'acl_users')
@@ -472,6 +473,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
                 fixOwnerRole(obj, user.getId())
                 if base_hasattr(obj, 'reindexObject'):
                     obj.reindexObject()
+    changeOwnershipOf = postonly(changeOwnershipOf)
 
     security.declarePublic('urlparse')
     def urlparse(self, url):
@@ -592,7 +594,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
     security.declarePublic('getInheritedLocalRoles')
     def getInheritedLocalRoles(self, here):
         """Returns a tuple with the acquired local roles."""
-        portal = here.portal_url.getPortalObject()
+        portal = getToolByName(here, 'portal_url').getPortalObject()
         result = []
         cont = 1
         if portal != here:
@@ -600,7 +602,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
             while cont:
                 if not getattr(parent, 'acl_users', False):
                     break
-                userroles = parent.acl_users.getLocalRolesForDisplay(parent)
+                userroles = parent.acl_users._getLocalRolesForDisplay(parent)
                 for user, roles, role_type, name in userroles:
                     # Find user in result
                     found = 0
@@ -939,7 +941,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
             return True
 
     security.declarePublic('acquireLocalRoles')
-    def acquireLocalRoles(self, obj, status = 1):
+    def acquireLocalRoles(self, obj, status = 1, REQUEST=None):
         """If status is 1, allow acquisition of local roles (regular behaviour).
 
         If it's 0, prohibit it (it will allow some kind of local role
@@ -954,6 +956,7 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
         gruf._acquireLocalRoles(obj, status)
         # Reindex the whole stuff.
         obj.reindexObjectSecurity()
+    acquireLocalRoles = postonly(acquireLocalRoles)
 
     security.declarePublic('isLocalRoleAcquired')
     def isLocalRoleAcquired(self, obj):
@@ -1227,4 +1230,116 @@ class PloneTool(PloneBaseTool, UniqueObject, SimpleItem):
         else:
             return None
 
+    # This is public because we don't know what permissions the user
+    # has on the objects to be deleted.  The restrictedTraverse and
+    # manage_delObjects calls should handle permission checks for us.
+    security.declarePublic('deleteObjectsByPaths')
+    def deleteObjectsByPaths(self, paths, handle_errors=True, REQUEST=None):
+        failure = {}
+        success = []
+        # use the portal for traversal in case we have relative paths
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        traverse = portal.restrictedTraverse
+        for path in paths:
+            # Skip and note any errors
+            if handle_errors:
+                sp = transaction.savepoint()
+            try:
+                obj = traverse(path)
+                obj_parent = aq_parent(aq_inner(obj))
+                obj_parent.manage_delObjects([obj.getId()])
+                success.append('%s (%s)' % (obj.title_or_id(), path))
+            except ConflictError:
+                raise
+            except Exception, e:
+                if handle_errors:
+                    sp.rollback()
+                    failure[path]= e
+                else:
+                    raise
+        transaction_note('Deleted %s' % (', '.join(success)))
+        return success, failure
+    deleteObjectsByPaths = postonly(deleteObjectsByPaths)
+
+    security.declarePublic('transitionObjectsByPaths')
+    def transitionObjectsByPaths(self, workflow_action, paths, comment='',
+                                 expiration_date=None, effective_date=None,
+                                 include_children=False, handle_errors=True,
+                                 REQUEST=None):
+        failure = {}
+        # use the portal for traversal in case we have relative paths
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        traverse = portal.restrictedTraverse
+        for path in paths:
+            if handle_errors:
+                sp = transaction.savepoint()
+            try:
+                o = traverse(path, None)
+                if o is not None:
+                    o.content_status_modify(workflow_action,
+                                            comment,
+                                            effective_date=effective_date,
+                                            expiration_date=expiration_date)
+            except ConflictError:
+                raise
+            except Exception, e:
+                if handle_errors:
+                    # skip this object but continue with sub-objects.
+                    sp.rollback()
+                    failure[path]= e
+                else:
+                    raise
+            if getattr(o, 'isPrincipiaFolderish', None) and include_children:
+                subobject_paths = ["%s/%s" % (path, id) for id in o.objectIds()]
+                self.transitionObjectsByPaths(workflow_action, subobject_paths,
+                                              comment, expiration_date,
+                                              effective_date, include_children,
+                                              handle_errors)
+        return failure
+    transitionObjectsByPaths = postonly(transitionObjectsByPaths)
+
+    security.declarePublic('renameObjectsByPaths')
+    def renameObjectsByPaths(self, paths, new_ids, new_titles,
+                             handle_errors=True, REQUEST=None):
+        failure = {}
+        success = {}
+        # use the portal for traversal in case we have relative paths
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        traverse = portal.restrictedTraverse
+        for i, path in enumerate(paths):
+            new_id = new_ids[i]
+            new_title = new_titles[i]
+            if handle_errors:
+                sp = transaction.savepoint()
+            try:
+                obj = traverse(path, None)
+                obid = obj.getId()
+                title = obj.Title()
+                change_title = new_title and title != new_title
+                changed = False
+                if change_title:
+                    obj.setTitle(new_title)
+                    changed = True
+                if new_id and obid != new_id:
+                    parent = aq_parent(aq_inner(obj))
+                    parent.manage_renameObjects((obid,), (new_id,))
+                    changed = True
+                elif change_title:
+                    # the rename will have already triggered a reindex
+                    obj.reindexObject()
+                if changed:
+                    success[path]=(new_id,new_title)
+            except ConflictError:
+                raise
+            except Exception, e:
+                if handle_errors:
+                    # skip this object but continue with sub-objects.
+                    sp.rollback()
+                    failure[path]= e
+                else:
+                    raise
+        transaction_note('Renamed %s' % str(success.keys))
+        return success, failure
+    renameObjectsByPaths = postonly(renameObjectsByPaths)
+
 InitializeClass(PloneTool)
