@@ -6,16 +6,14 @@ import os, sys
 if __name__ == '__main__':
     execfile(os.path.join(sys.path[0], 'framework.py'))
 
-from Testing import ZopeTestCase
 from Products.CMFPlone.tests import PloneTestCase
 
 from Acquisition import aq_base
 from Globals import REPLACEABLE
 from DateTime import DateTime
 from Products.CMFCore.permissions import AccessInactivePortalContent
-from Products.CMFPlone import transaction
+import transaction
 
-from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectRegistry
 from Products.CMFPlone.CatalogTool import ExtensibleIndexableObjectWrapper
 from Products.CMFPlone.CatalogTool import _eioRegistry
 
@@ -28,7 +26,8 @@ default_user  = PloneTestCase.default_user
 user2  = 'u2'
 group2 = 'g2'
 
-base_content = ['Members', 'events', 'news', 'previous', default_user, 'doc']
+base_content = ['Members', 'events', 'news', 'previous',
+                default_user, 'front-page', 'doc']
 
 
 class TestCatalogSetup(PloneTestCase.PloneTestCase):
@@ -151,40 +150,48 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Document', id='doc', title='Foo', description='Bar')
         self.catalog.unindexObject(self.folder.doc)
 
+    def assertResults(self, result, expect):
+        # Verifies ids of catalog results against expected ids
+        lhs = [r.getId for r in result]
+        lhs.sort()
+        rhs = list(expect)
+        rhs.sort()
+        self.assertEqual(lhs, rhs)
+
     def testFixture(self):
         self.assertEqual(self.folder.doc.getId(), 'doc')
         self.assertEqual(self.folder.doc.Title(), 'Foo')
         self.assertEqual(self.folder.doc.Description(), 'Bar')
-        self.assertEqual(len(self.catalog(id='doc')), 0)
+        self.assertEqual(len(self.catalog(getId='doc')), 0)
         self.assertEqual(len(self.catalog(Title='Foo')), 0)
         self.assertEqual(len(self.catalog(Description='Bar')), 0)
 
     def testIndexObject(self):
         # Object should be indexed
         self.catalog.indexObject(self.folder.doc)
-        self.assertEqual(len(self.catalog(id='doc')), 1)
+        self.assertEqual(len(self.catalog(getId='doc')), 1)
         self.assertEqual(len(self.catalog(Title='Foo')), 1)
         self.assertEqual(len(self.catalog(Description='Bar')), 1)
 
     def testReindexObject(self):
         # Object should be indexed
         self.catalog.reindexObject(self.folder.doc)
-        self.assertEqual(len(self.catalog(id='doc')), 1)
+        self.assertEqual(len(self.catalog(getId='doc')), 1)
         self.assertEqual(len(self.catalog(Title='Foo')), 1)
         self.assertEqual(len(self.catalog(Description='Bar')), 1)
 
     def testUnindexObject(self):
         # Object should be unindexed
         self.catalog.indexObject(self.folder.doc)
-        self.assertEqual(len(self.catalog(id='doc')), 1)
+        self.assertEqual(len(self.catalog(getId='doc')), 1)
         self.catalog.unindexObject(self.folder.doc)
-        self.assertEqual(len(self.catalog(id='doc')), 0)
+        self.assertEqual(len(self.catalog(getId='doc')), 0)
 
     def testIndexObjectUpdatesMetadata(self):
         # Indexing should update metadata
         self.catalog.indexObject(self.folder.doc)
-        brain = self.catalog(id='doc')[0]
-        self.assertEqual(brain.id, 'doc')
+        brain = self.catalog(getId='doc')[0]
+        self.assertEqual(brain.getId, 'doc')
         self.assertEqual(brain.Title, 'Foo')
         self.assertEqual(brain.Description, 'Bar')
 
@@ -194,8 +201,8 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.doc.setTitle('Fred')
         self.folder.doc.setDescription('BamBam')
         self.catalog.reindexObject(self.folder.doc)
-        brain = self.catalog(id='doc')[0]
-        self.assertEqual(brain.id, 'doc')
+        brain = self.catalog(getId='doc')[0]
+        self.assertEqual(brain.getId, 'doc')
         self.assertEqual(brain.Title, 'Fred')
         self.assertEqual(brain.Description, 'BamBam')
 
@@ -205,9 +212,9 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.doc.setTitle('Fred')
         self.folder.doc.setDescription('BamBam')
         self.catalog.reindexObject(self.folder.doc, update_metadata=0)
-        brain = self.catalog(id='doc')[0]
+        brain = self.catalog(getId='doc')[0]
         # Metadata did not change
-        self.assertEqual(brain.id, 'doc')
+        self.assertEqual(brain.getId, 'doc')
         self.assertEqual(brain.Title, 'Foo')
         self.assertEqual(brain.Description, 'Bar')
 
@@ -217,7 +224,7 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.doc.setTitle('Fred')
         self.folder.doc.setDescription('BamBam')
         self.catalog.reindexObject(self.folder.doc, idxs=['Title'])
-        self.assertEqual(len(self.catalog(id='doc')), 1)
+        self.assertEqual(len(self.catalog(getId='doc')), 1)
         self.assertEqual(len(self.catalog(Title='Fred')), 1)
         # Description index did not change
         self.assertEqual(len(self.catalog(Description='Bar')), 1)
@@ -229,8 +236,8 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.doc.setTitle('Fred')
         self.folder.doc.setDescription('BamBam')
         self.catalog.reindexObject(self.folder.doc, idxs=['Title'])
-        brain = self.catalog(id='doc')[0]
-        self.assertEqual(brain.id, 'doc')
+        brain = self.catalog(getId='doc')[0]
+        self.assertEqual(brain.getId, 'doc')
         self.assertEqual(brain.Title, 'Fred')
         self.assertEqual(brain.Description, 'BamBam')
 
@@ -240,16 +247,16 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         self.folder.doc.setTitle('Fred')
         self.folder.doc.setDescription('BamBam')
         self.catalog.reindexObject(self.folder.doc, idxs=['Title'], update_metadata=0)
-        brain = self.catalog(id='doc')[0]
+        brain = self.catalog(getId='doc')[0]
         # Metadata did not change
-        self.assertEqual(brain.id, 'doc')
+        self.assertEqual(brain.getId, 'doc')
         self.assertEqual(brain.Title, 'Foo')
         self.assertEqual(brain.Description, 'Bar')
 
     def testIndexTitleOnly(self):
         # Indexing should only index the Title
         #
-        # XXX: This does not work as expected. The object
+        # TODO: This does not work as expected. The object
         # appears to be in the catalog but is not returned
         # by searchResults()!?
         #
@@ -258,25 +265,65 @@ class TestCatalogIndexing(PloneTestCase.PloneTestCase):
         path = self.catalog._CatalogTool__url(self.folder.doc)
         self.failUnless(path in self.catalog._catalog.paths.values())
         # But it is not returned when searching...
-        self.assertEqual(len(self.catalog(id='doc')), 0)
+        self.assertEqual(len(self.catalog(getId='doc')), 0)
         self.assertEqual(len(self.catalog(Title='Foo')), 0) # <-- Should be 1
         self.assertEqual(len(self.catalog(Description='Bar')), 0)
 
     def testIndexIdOnly(self):
         # Indexing should only index the id
         #
-        # XXX: Demonstrate that the behavior is independent
+        # TODO: Demonstrate that the behavior is independent
         # of index type.
         #
-        self.catalog.indexObject(self.folder.doc, idxs=['id'])
+        self.catalog.indexObject(self.folder.doc, idxs=['getId'])
         # The document is cataloged
         path = self.catalog._CatalogTool__url(self.folder.doc)
         self.failUnless(path in self.catalog._catalog.paths.values())
         # But it is not returned when searching...
-        self.assertEqual(len(self.catalog(id='doc')), 0) # <-- Should be 1
+        self.assertEqual(len(self.catalog(getId='doc')), 0) # <-- Should be 1
         self.assertEqual(len(self.catalog(Title='Foo')), 0)
         self.assertEqual(len(self.catalog(Description='Bar')), 0)
 
+    def testClearFindAndRebuildRemovesBadContent(self):
+        # Index the doc for consistency
+        self.catalog.indexObject(self.folder.doc)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+        # index an object which shouldn't be there
+        self.catalog.indexObject(self.portal.portal_skins)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content+['portal_skins'])
+        self.catalog.clearFindAndRebuild()
+        # This will remove the extraneous item and add the document added
+        # in afterSetup
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+
+    def testClearFindAndRebuildAddsMissingContent(self):
+        # Index the doc for consistency
+        self.catalog.indexObject(self.folder.doc)
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+        # index an object which shouldn't be there
+        self.catalog.unindexObject(self.portal.Members)
+        altered_content = base_content[:]
+        altered_content.remove('Members')
+        res = self.catalog.searchResults()
+        self.assertResults(res, altered_content)
+        self.catalog.clearFindAndRebuild()
+        # This will add the missing item and also the document added
+        # in afterSetup
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+
+    def testClearFindAndRebuildKeepsModificationDate(self):
+        # Index the doc for consistency
+        self.catalog.indexObject(self.folder.doc)
+        self.folder.doc.setModificationDate(DateTime(0))
+        self.catalog.clearFindAndRebuild()
+        self.assertEquals(self.folder.doc.modified(), DateTime(0))
+        self.assertEquals(len(self.catalog(modified=DateTime(0))), 1)
+
 
 class TestCatalogSearching(PloneTestCase.PloneTestCase):
 
@@ -285,7 +332,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.workflow = self.portal.portal_workflow
         self.groups = self.portal.portal_groups
 
-        self.portal.acl_users._doAddUser(user2, 'secret', [], [], [])
+        self.portal.acl_users._doAddUser(user2, 'secret', [], [])
 
         self.folder.invokeFactory('Document', id='doc', text='foo')
         self.folder.invokeFactory('Folder', id='folder2')
@@ -302,9 +349,10 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.groups.groupWorkspacesCreationFlag = 0
         self.groups.addGroup(group2, None, [], [])
         group = self.groups.getGroupById(group2)
+        self.loginAsPortalOwner() # GRUF 3.52
         group.addMember(user2)
-        prefix = self.portal.acl_users.getGroupPrefix()
-        return '%s%s' % (prefix, group2)
+        self.login(default_user) # Back to normal
+        return group2
 
     def testListAllowedRolesAndUsers(self):
         # Should include the group in list of allowed users
@@ -315,7 +363,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
 
     def testSearchReturnsDocument(self):
         # Document should be found when owner does a search
-        self.assertEqual(self.catalog(SearchableText='foo')[0].id, 'doc')
+        self.assertEqual(self.catalog(SearchableText='aaa')[0].id, 'aaa')
 
     def testSearchDoesNotReturnDocument(self):
         # Document should not be found when user2 does a search
@@ -333,7 +381,7 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         self.assertEqual(len(self.catalog(SearchableText='aaa AND ccc')), 1)
 
     def testSearchReturnsDocumentUsing_OR(self):
-        # Two documents (aaa, bbb)  should be found when owner does a search using OR 
+        # Two documents (aaa, bbb)  should be found when owner does a search using OR
         results = self.catalog(SearchableText='aaa OR bbb')
         self.assertEqual(len(results), 2)
 
@@ -341,29 +389,34 @@ class TestCatalogSearching(PloneTestCase.PloneTestCase):
         # After adding a group with access rights and containing user2,
         # a search must find the document.
         groupname = self.addUser2ToGroup()
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
+        self.app.REQUEST.set('REQUEST_METHOD', 'GET')
         self.login(user2)
-        self.assertEqual(self.catalog(SearchableText='foo')[0].id, 'doc')
+        self.assertEqual(self.catalog(SearchableText='aaa')[0].id, 'aaa')
 
     def testSearchRespectsLocalRoleAcquisition(self):
         # After adding a group with access rights and containing user2,
         # a search must find the document in subfolders.
         groupname = self.addUser2ToGroup()
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
+        self.app.REQUEST.set('REQUEST_METHOD', 'GET')
         self.login(user2)
         # Local Role works in subfolder
-        self.assertEqual(self.catalog(SearchableText='bar')[0].id, 'doc2')
+        self.assertEqual(self.catalog(SearchableText='bbb')[0].id, 'bbb')
 
     def testSearchRespectsLocalRoleAcquisitionDisabled(self):
         # After adding a group with access rights and containing user2,
         # a search should not find documents in subfolders which have
         # disabled local role acquisition.
         groupname = self.addUser2ToGroup()
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_localrole_edit('add', [groupname], 'Owner')
         # Acquisition off for folder2
-        self.login(default_user)
         self.folder.folder2.folder_localrole_set(use_acquisition=0)
-        #Everything in subfolder should be invisible
+        self.app.REQUEST.set('REQUEST_METHOD', 'GET')
+        # Everything in subfolder should be invisible
         self.login(user2)
         self.failIf(self.catalog(SearchableText='bar'))
 
@@ -410,7 +463,7 @@ class TestCatalogSorting(PloneTestCase.PloneTestCase):
     def testSortableTitleOutput(self):
         doc = self.folder.doc
         wrapped = ExtensibleIndexableObjectWrapper(vars, doc, self.portal)
-        
+
         self.assertEqual(wrapped.sortable_title, u'00000012 document 00000025')
 
     def testSortableNonASCIITitles(self):
@@ -423,7 +476,7 @@ class TestCatalogSorting(PloneTestCase.PloneTestCase):
 
 
 class TestFolderCataloging(PloneTestCase.PloneTestCase):
-    # Tests for http://plone.org/collector/2876
+    # Tests for http://dev.plone.org/plone/ticket/2876
     # folder_edit must recatalog. folder_rename must recatalog.
 
     def afterSetUp(self):
@@ -431,55 +484,57 @@ class TestFolderCataloging(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('Folder', id='foo')
 
     def testFolderTitleIsUpdatedOnEdit(self):
-        # Test for catalog that searches to ensure folder titles are 
-        # updated in the catalog. 
+        # Test for catalog that searches to ensure folder titles are
+        # updated in the catalog.
         title = 'Test Folder - Snooze!'
         self.folder.foo.folder_edit(title, '')
         results = self.catalog(Title='Snooze')
         self.failUnless(results)
         for result in results:
             self.assertEqual(result.Title, title)
-            self.assertEqual(result.id, 'foo')
+            self.assertEqual(result.getId, 'foo')
 
     def testFolderTitleIsUpdatedOnRename(self):
-        # Test for catalog that searches to ensure folder titles are 
-        # updated in the catalog. 
+        # Test for catalog that searches to ensure folder titles are
+        # updated in the catalog.
         title = 'Test Folder - Snooze!'
-        transaction.commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.foo.folder_edit(title, '', id='bar')
         results = self.catalog(Title='Snooze')
         self.failUnless(results)
         for result in results:
             self.assertEqual(result.Title, title)
-            self.assertEqual(result.id, 'bar')
+            self.assertEqual(result.getId, 'bar')
 
     def testFolderTitleIsUpdatedOnFolderTitleChange(self):
         # The bug in fact talks about folder_rename
         title = 'Test Folder - Snooze!'
         foo_path = '/'.join(self.folder.foo.getPhysicalPath())
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_rename(paths=[foo_path], new_ids=['foo'], new_titles=[title])
         results = self.catalog(Title='Snooze')
         self.failUnless(results)
         for result in results:
             self.assertEqual(result.Title, title)
-            self.assertEqual(result.id, 'foo')
+            self.assertEqual(result.getId, 'foo')
 
     def testFolderTitleIsUpdatedOnFolderRename(self):
         # The bug in fact talks about folder_rename
         title = 'Test Folder - Snooze!'
-        transaction.commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         foo_path = '/'.join(self.folder.foo.getPhysicalPath())
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_rename(paths=[foo_path], new_ids=['bar'], new_titles=[title])
         results = self.catalog(Title='Snooze')
         self.failUnless(results)
         for result in results:
             self.assertEqual(result.Title, title)
-            self.assertEqual(result.id, 'bar')
+            self.assertEqual(result.getId, 'bar')
 
     def testSetTitleDoesNotUpdateCatalog(self):
         # setTitle() should not update the catalog
         title = 'Test Folder - Snooze!'
-        self.failUnless(self.catalog(id='foo'))
+        self.failUnless(self.catalog(getId='foo'))
         self.folder.foo.setTitle(title)
         #Title is a TextIndex
         self.failIf(self.catalog(Title='Snooze'))
@@ -505,7 +560,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc2','doc1','doc3','doc4']
+        expected = ['doc2','doc1','doc3','doc4', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsUpdatedOnMoveUp(self):
@@ -513,7 +568,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc1','doc3','doc2','doc4']
+        expected = ['doc1','doc3','doc2','doc4', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsUpdatedOnMoveTop(self):
@@ -521,7 +576,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc3','doc1','doc2','doc4']
+        expected = ['doc3','doc1','doc2','doc4', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsUpdatedOnMoveBottom(self):
@@ -529,7 +584,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc1','doc2','doc4','doc3']
+        expected = ['doc1','doc2','doc4','doc3', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsFineWithObjectCreation(self):
@@ -537,7 +592,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc1','doc2','doc3','doc4','doc5']
+        expected = ['doc1','doc2','doc3','doc4','doc5', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsFineWithObjectDeletion(self):
@@ -545,7 +600,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc1','doc2','doc4']
+        expected = ['doc1','doc2','doc4', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderIsFineWithObjectRenaming(self):
@@ -554,13 +609,13 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         # that blames permissions or lack of support by the obj. The obj is a
         # Plone Document, and the owner of doc2 is portal_owner. Harumph.
 
-        transaction.commit(1)
+        transaction.savepoint(optimistic=True)
 
         self.folder.manage_renameObjects(['doc2'], ['buzz'])
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
                                    sort_on = 'getObjPositionInParent')
-        expected = ['doc1','buzz','doc3','doc4']
+        expected = ['doc1','buzz','doc3','doc4', 'front-page']
         self.failUnlessEqual([b.getId for b in folder_docs], expected)
 
     def testOrderAfterALotOfChanges(self):
@@ -582,7 +637,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         #self.folder.manage_renameObjects('Document', id='doc5', text='blam')
 
         self.folder.manage_delObjects(['doc3','doc4','doc5','doc7'])
-        expected = ['doc2','doc1','doc6','doc8']
+        expected = ['doc2','doc1','doc6','doc8', 'front-page']
 
         folder_docs = self.catalog(portal_type = 'Document',
                                    path = self.folder.getPhysicalPath(),
@@ -596,7 +651,7 @@ class TestCatalogOrdering(PloneTestCase.PloneTestCase):
         all_objs = self.catalog()
         sorted_objs = self.catalog(sort_on='getObjPositionInParent')
         self.failUnlessEqual(len(all_objs), len(sorted_objs))
-        
+
         members = self.portal.Members
         members_path = '/'.join(members.getPhysicalPath())
         members_query = self.catalog(path=members_path)
@@ -614,7 +669,7 @@ class TestCatalogBugs(PloneTestCase.PloneTestCase):
         # Should be able to copy/paste a portal containing
         # a catalog tool. Triggers manage_afterAdd of portal_catalog
         # thereby exposing a bug which is now going to be fixed.
-        self.loginPortalOwner()
+        self.loginAsPortalOwner()
         cb = self.app.manage_copyObjects([portal_name])
         self.app.manage_pasteObjects(cb)
         self.failUnless(hasattr(self.app, 'copy_of_'+portal_name))
@@ -628,17 +683,29 @@ class TestCatalogBugs(PloneTestCase.PloneTestCase):
         self.folder.manage_pasteObjects(cb)
         self.failUnless(hasattr(aq_base(self.folder), 'portal_catalog'))
 
+    def testPastingCatalogPreservesTextIndexes(self):
+        # Pasting the catalog should not cause indexes to be removed.
+        self.setRoles(['Manager'])
+        self.catalog.__replaceable__ = REPLACEABLE
+        cb = self.portal.manage_copyObjects(['portal_catalog'])
+        self.folder.manage_pasteObjects(cb)
+        self.failUnless(hasattr(aq_base(self.folder), 'portal_catalog'))
+        cat = self.folder.portal_catalog
+        self.failUnless('SearchableText' in cat.indexes())
+        # CMF added lexicons should stick around too
+        self.failUnless(hasattr(aq_base(cat), 'plaintext_lexicon'))
+
     def testCanRenamePortalIfLexiconExists(self):
         # Should be able to rename a Plone portal
-        # This test is to demonstrate that http://plone.org/collector/1745
+        # This test is to demonstrate that http://dev.plone.org/plone/ticket/1745
         # is fixed and can be closed.
-        self.loginPortalOwner()
+        self.loginAsPortalOwner()
         self.app.manage_renameObjects([portal_name], ['foo'])
         self.failUnless(hasattr(self.app, 'foo'))
 
 
 class TestCatalogUnindexing(PloneTestCase.PloneTestCase):
-    # Tests for http://plone.org/collector/3547
+    # Tests for http://dev.plone.org/plone/ticket/3547
     # Published objects are not unindexed on delete?
 
     def afterSetUp(self):
@@ -651,41 +718,41 @@ class TestCatalogUnindexing(PloneTestCase.PloneTestCase):
         self.assertEqual(state, 'visible')
 
     def testVisibleCanBeFound(self):
-        self.failUnless(self.catalog(id='doc'))
+        self.failUnless(self.catalog(getId='doc'))
 
     def testVisibleIsUnindexed(self):
         self.folder._delObject('doc')
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
     def testPrivateCanBeFound(self):
         self.workflow.doActionFor(self.folder.doc, 'hide')
-        self.failUnless(self.catalog(id='doc'))
+        self.failUnless(self.catalog(getId='doc'))
 
     def testPrivateIsUnindexed(self):
         self.workflow.doActionFor(self.folder.doc, 'hide')
         self.folder._delObject('doc')
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
     def testPendingCanBeFound(self):
         self.workflow.doActionFor(self.folder.doc, 'submit')
-        self.failUnless(self.catalog(id='doc'))
+        self.failUnless(self.catalog(getId='doc'))
 
     def testPendingIsUnindexed(self):
         self.workflow.doActionFor(self.folder.doc, 'submit')
         self.folder._delObject('doc')
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
     def testPublishedCanBeFound(self):
         self.setRoles(['Manager'])
         self.workflow.doActionFor(self.folder.doc, 'publish')
-        self.failUnless(self.catalog(id='doc'))
+        self.failUnless(self.catalog(getId='doc'))
 
     def testPublishedIsUnindexed(self):
         # Works here!
         self.setRoles(['Manager'])
         self.workflow.doActionFor(self.folder.doc, 'publish')
         self.folder._delObject('doc')
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
     def testPublishedIsUnindexedIfOwnerDeletes(self):
         # Works here!
@@ -693,7 +760,7 @@ class TestCatalogUnindexing(PloneTestCase.PloneTestCase):
         self.workflow.doActionFor(self.folder.doc, 'publish')
         self.setRoles(['Member'])
         self.folder._delObject('doc')
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
     def testPublishedIsUnindexedByFolderDeleteScript(self):
         # Works here too!
@@ -702,8 +769,11 @@ class TestCatalogUnindexing(PloneTestCase.PloneTestCase):
         self.setRoles(['Member'])
         doc_path = '/'.join(self.folder.doc.getPhysicalPath())
         self.app.REQUEST.set('paths', [doc_path])
+        # folder_delete requires a non-GET request
+        self.app.REQUEST.set('REQUEST_METHOD', 'POST')
         self.folder.folder_delete()
-        self.failIf(self.catalog(id='doc'))
+        self.app.REQUEST.set('REQUEST_METHOD', 'GET')
+        self.failIf(self.catalog(getId='doc'))
 
     def testPublishedIsUnindexedWhenDeletingParentFolder(self):
         # Works here too!
@@ -711,7 +781,7 @@ class TestCatalogUnindexing(PloneTestCase.PloneTestCase):
         self.workflow.doActionFor(self.folder.doc, 'publish')
         self.setRoles(['Member'])
         self.folder.aq_parent._delObject(self.folder.getId())
-        self.failIf(self.catalog(id='doc'))
+        self.failIf(self.catalog(getId='doc'))
 
 
 class TestCatalogExpirationFiltering(PloneTestCase.PloneTestCase):
@@ -789,6 +859,35 @@ class TestCatalogExpirationFiltering(PloneTestCase.PloneTestCase):
         res = self.catalog()
         self.assertResults(res, base_content)
 
+    def testSearchResultsWithAdditionalExpiryFilter(self):
+        # For this test we want the expires and effective indices in place,
+        # let's make sure everything still works
+        self.folder.doc.setExpirationDate(DateTime(2000, 12, 31))
+        self.folder.doc.reindexObject()
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content[:-1])
+        # Now make the object expire at some fixed date in the future
+        self.folder.doc.setExpirationDate(DateTime()+2)
+        self.folder.doc.reindexObject()
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content)
+        # We should be able to further limit the search using the exipres
+        # and efective indices.
+        res = self.catalog.searchResults(expires={'query':DateTime()+3,
+                                                  'range':'min'})
+        self.assertResults(res, base_content[:-1])
+
+    def testSearchResultsExpiredWithAdditionalExpiryFilter(self):
+        # Now make the object expire at some date in the recent past
+        self.folder.doc.setExpirationDate(DateTime()-2)
+        self.folder.doc.reindexObject()
+        res = self.catalog.searchResults()
+        self.assertResults(res, base_content[:-1])
+        # Even if we explicitly ask for it, we shouldn't get expired content
+        res = self.catalog.searchResults(expires={'query':DateTime()-3,
+                                                  'range':'min'})
+        self.assertResults(res, base_content[:-1])
+
 
 def dummyMethod(obj, **kwargs):
     return 'a dummy'
@@ -796,17 +895,17 @@ def dummyMethod(obj, **kwargs):
 class TestExtensibleIndexableObjectWrapper(PloneTestCase.PloneTestCase):
     """Tests for the wrapper
     """
-    
+
     def afterSetUp(self):
         self.folder.invokeFactory('Document', 'doc', title='document')
         self.doc = self.folder.doc
         _eioRegistry.register('dummy', dummyMethod)
-        
+
     def testSetup(self):
         doc = self.doc
         self.failUnlessEqual(doc.getId(), 'doc')
         self.failUnlessEqual(doc.Title(), 'document')
-        
+
     def testWrapper(self):
         doc = self.doc
         vars = {'var' : 'a var'}
@@ -815,7 +914,7 @@ class TestExtensibleIndexableObjectWrapper(PloneTestCase.PloneTestCase):
         self.failUnlessEqual(wrapped.getId(), 'doc')
         self.failUnlessEqual(wrapped.Title(), 'document')
         self.failUnlessEqual(wrapped.dummy, 'a dummy')
-        
+
     def beforeTearDown(self):
         _eioRegistry.unregister('dummy')
 
@@ -824,13 +923,23 @@ class TestExtensibleIndexableObjectWrapper(PloneTestCase.PloneTestCase):
         self.failIf(is_folderish(i))
 
     def test_is_folderishWithFolder(self):
-        f = dummy.ATFolder('struct_folder')
+        f = dummy.Folder('struct_folder')
         self.failUnless(is_folderish(f))
 
     def test_is_folderishWithNonStructuralFolder(self):
         f = dummy.NonStructuralFolder('ns_folder')
         self.failIf(is_folderish(f))
 
+    def test_provided(self):
+        from Products.CMFCore.interfaces import IContentish
+        from Products.CMFCore.interfaces import IIndexableObjectWrapper
+        from Products.CMFCore.tests.base.dummy import DummyContent
+
+        obj = DummyContent()
+        w = ExtensibleIndexableObjectWrapper({}, obj, self.portal)
+        self.failUnless(IContentish.providedBy(w))
+        self.failUnless(IIndexableObjectWrapper.providedBy(w))
+
 
 def test_suite():
     from unittest import TestSuite, makeSuite
