@@ -1,7 +1,9 @@
 from AccessControl import ClassSecurityInfo
 from Globals import InitializeClass
+from ZODB.POSException import ConflictError
 from OFS.SimpleItem import SimpleItem
 from Products.CMFCore.Expression import Expression
+from zope.interface import implements
 # BBB CMF < 1.5
 try:
     from Products.CMFCore.permissions import ManagePortal
@@ -16,6 +18,8 @@ from Products.CMFCore.utils import UniqueObject, getToolByName
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from ZPublisher import BeforeTraverse
 from ZPublisher.HTTPRequest import HTTPRequest
+from Products.PloneLanguageTool.tldmap import tld_to_language
+from Products.PloneLanguageTool.interfaces import ILanguageTool
 
 try:
     from Products.CMFPlone.interfaces.Translatable import ITranslatable
@@ -28,11 +32,17 @@ try:
 except ImportError:
     _hasPTS = 0
 
+import logging
+logger = logging.getLogger('PloneLanguageTool')
+
 import availablelanguages
 
+
 class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
     """Language Administration Tool For Plone."""
 
+    implements(ILanguageTool)
+
     id  = 'portal_languages'
     title = 'Manages available languages'
     meta_type = 'Plone Language Tool'
@@ -44,8 +54,10 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
     local_available_countries = {}
 
     use_path_negotiation = 1
+    use_content_negotiation = 1
     use_cookie_negotiation = 1
     use_request_negotiation = 1
+    use_cctld_negotiation = 0
     use_combined_language_codes = 0
     display_flags = 1
     start_neutral = 1
@@ -75,6 +87,7 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
         self.use_path_negotiation = 1
         self.use_cookie_negotiation  = 1
         self.use_request_negotiation = 1
+        self.use_cctld_negotiation = 0
         self.force_language_urls = 1
         self.allow_content_language_fallback = 0
         self.display_flags = 1
@@ -93,11 +106,13 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
 
     security.declareProtected(ManagePortal, 'manage_setLanguageSettings')
     def manage_setLanguageSettings(self, defaultLanguage, supportedLanguages,
+                                   setContentN=None,
                                    setCookieN=None, setRequestN=None,
                                    setPathN=None, setForcelanguageUrls=None,
                                    setAllowContentLanguageFallback=None,
                                    setUseCombinedLanguageCodes=None,
                                    displayFlags=None, startNeutral=None,
+                                   setCcTLDN=None,
                                    REQUEST=None):
         """Stores the tool settings."""
         if supportedLanguages and type(supportedLanguages) == type([]):
@@ -108,6 +123,11 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
         else:
             self.setDefaultLanguage(self.supported_langs[0])
 
+        if setContentN:
+            self.use_content_negotiation = 1
+        else:
+            self.use_content_negotiation = 0
+
         if setCookieN:
             self.use_cookie_negotiation = 1
         else:
@@ -123,6 +143,11 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
         else:
             self.use_path_negotiation = 0
 
+        if setCcTLDN:
+            self.use_cctld_negotiation = 1
+        else:
+            self.use_cctld_negotiation = 0
+
         if setForcelanguageUrls:
             self.force_language_urls = 1
         else:
@@ -253,12 +278,16 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
 
         This can override predefined ones.
         """
+        logger.log(logging.WARNING, 'Deprecation Warning: The addLanguage '
+            'method is deprecated and will be removed in PLT 2.0.')
         self.local_available_langs[langCode] = langDescription
         self._p_changed = 1
 
     security.declareProtected(ManagePortal, 'removeLanguage')
     def removeLanguage(self, langCode):
         """Removes a custom language from the tool."""
+        logger.log(logging.WARNING, 'Deprecation Warning: The removeLanguage '
+            'method is deprecated and will be removed in PLT 2.0.')
         if langCode in self.local_available_langs:
             del self.local_available_langs[langCode]
             self._p_changed = 1
@@ -349,19 +378,53 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
         # XXX Why this try/except?
         try:
             for item in items:
-                if len(item) == 2:
-                    if item in self.getSupportedLanguages():
-                        return item
+                if item in self.getSupportedLanguages():
+                    return item
+        except ConflictError:
+            raise
         except:
             pass
         return None
 
+    security.declarePublic('getContentLanguage')
+    def getContentLanguage(self):
+        """Checks the language of the current content if not folderish."""
+        request = getattr(self, 'REQUEST', None)
+        if request is None:
+            return None
+        try: # This will actually work nicely with browserdefault as we get attribute error...
+            contentpath = request.get('PATH_TRANSLATED')
+            # We have to filter the tools, especially portal_factory and the css and js registries
+            if contentpath is not None and contentpath.find('portal_') == -1:
+                obj = self.unrestrictedTraverse(contentpath, None)
+                if obj is not None:
+                    if obj.Language() in self.getSupportedLanguages():
+                        return obj.Language()
+        except ConflictError:
+            raise
+        except:
+            pass
+        return None
+
+    security.declareProtected(View, 'getCcTLDLanguages')
+    def getCcTLDLanguages(self):
+        if not hasattr(self, 'REQUEST'):
+            return None
+        request = self.REQUEST
+        if not "HTTP_HOST" in request:
+            return None
+        host=request["HTTP_HOST"].split(":")[0].lower()
+        tld=host.split(".")[-1]
+        wanted = tld_to_language.get(tld, [])
+        allowed = self.getSupportedLanguages()
+        return [lang for lang in wanted if lang in allowed]
+
     security.declareProtected(View, 'getRequestLanguages')
     def getRequestLanguages(self):
         """Parses the request and return language list."""
 
         if not hasattr(self, 'REQUEST'):
-            return []
+            return None
 
         # Get browser accept languages
         browser_pref_langs = self.REQUEST.get('HTTP_ACCEPT_LANGUAGE', '')
@@ -420,6 +483,8 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
     security.declareProtected(View, 'setLanguageBindings')
     def setLanguageBindings(self):
         """Setups the current language stuff."""
+        useContent = self.use_content_negotiation
+        useCcTLD = self.use_cctld_negotiation
         usePath = self.use_path_negotiation
         useCookie = self.use_cookie_negotiation
         useRequest = self.use_request_negotiation
@@ -431,7 +496,7 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
             # Create new binding instance
             binding = LanguageBinding(self)
         # Bind languages
-        lang = binding.setLanguageBindings(usePath, useCookie, useRequest, useDefault)
+        lang = binding.setLanguageBindings(usePath, useContent, useCookie, useRequest, useDefault, useCcTLD)
         # Set LANGUAGE to request
         self.REQUEST['LANGUAGE'] = lang
         # Set bindings instance to request
@@ -485,12 +550,16 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
 
         This can override predefined ones.
         """
+        logger.log(logging.WARNING, 'Deprecation Warning: The addCountry '
+            'method is deprecated and will be removed in PLT 2.0.')
         self.local_available_countries[countryCode] = countryDescription
         self._p_changed = 1
 
     security.declareProtected(ManagePortal, 'removeCountry')
     def removeCountry(self, countryCode):
         """Removes a custom country from the tool."""
+        logger.log(logging.WARNING, 'Deprecation Warning: The removeCountry '
+            'method is deprecated and will be removed in PLT 2.0.')
         if countryCode in self.local_available_countries:
             del self.local_available_countries[countryCode]
             self._p_changed = 1
@@ -509,14 +578,20 @@ class LanguageBinding:
         self.tool = tool
 
     security.declarePrivate('setLanguageBindings')
-    def setLanguageBindings(self, usePath=1, useCookie=1, useRequest=1, useDefault=1):
+    def setLanguageBindings(self, usePath=1, useContent=1, useCookie=1, useRequest=1, useDefault=1, useCcTLD=0):
         """Setup the current language stuff."""
         langs = []
 
         if usePath:
             # This one is set if there is an allowed language in the current path
             langsPath = [self.tool.getPathLanguage()]
-        else:langsPath = []
+        else:
+            langsPath = []
+
+        if useContent:
+            langsContent = [self.tool.getContentLanguage()]
+        else:
+            langsContent = []
 
         if useCookie:
             # If we are using the cookie stuff we provide the setter here
@@ -526,7 +601,13 @@ class LanguageBinding:
             else:
                 # Get from cookie
                 langsCookie = [self.tool.getLanguageCookie()]
-        else: langsCookie = []
+        else:
+            langsCookie = []
+
+        if useCcTLD:
+            langsCcTLD = self.tool.getCcTLDLanguages()
+        else:
+            langsCcTLD = []
 
         # Get langs from request
         if useRequest:
@@ -541,10 +622,14 @@ class LanguageBinding:
             langsDefault = []
 
         # Build list
-        langs = langsPath + langsCookie + langsRequest + langsDefault
+        langs = langsPath+langsContent+langsCookie+langsCcTLD+langsRequest+langsDefault
 
         # Filter None languages
-        langs = filter(lambda x: x is not None, langs)
+        langs = [lang for lang in langs if lang is not None]
+
+        # Set cookie language to language
+        if useCookie and langs[0] != langsCookie:
+            self.tool.setLanguageCookie(langs[0], noredir=True)
 
         self.DEFAULT_LANGUAGE = langs[-1]
         self.LANGUAGE = langs[0]
